#!/usr/bin/perl

use strict;
use Test::More;

my $nbtest;

# BEGIN { use_ok(...); } is not at the end, after the "plan"

#$FDN::Sql::DB_Access::debug = 1;
my $test = \&FDN::Sql::DB_Access::__parse_condition;
my @tables;
my $cond;
my $res;

@tables = ();
$cond = 'TABLE.TABLE_FIELD ~ "x%"';
$res = &$test($cond, \@tables);
is ( $res, 'TABLE.TABLE_FIELD LIKE "x%"', '__parse_condition');
is ( $#tables, 0, '__parse_condition');
is ( $tables[0], 'TABLE', '__parse_condition');
BEGIN { $nbtest += 3; }

@tables = ( 'PREVIOUS' );
$cond = 'TABLE_FIELD = PREVIOUS_VAL';
$res = &$test($cond,\@tables);
is ( $res, 'TABLE.TABLE_FIELD = PREVIOUS.PREVIOUS_VAL', '__parse_condition');
is ( $#tables, 1, '__parse_condition');
is ( $tables[0], 'PREVIOUS', '__parse_condition');
is ( $tables[1], 'TABLE', '__parse_condition');
BEGIN { $nbtest += 4; }

@tables = ();
$cond = 'TABLE_FIELD ~ "%%"';
$res = &$test($cond,\@tables);
is ( $res, 'none', '__parse_condition');
BEGIN { $nbtest += 1; }

$test = \&FDN::Sql::DB_Access::__parse_conditions;
my @conds;
my $exp;
@tables = ();
@conds = (
	'TABLE_FIELD = PREVIOUS_VAL or TABLE_FIELD = OTHER.PREVIOUS_OTHER',
	'TABLE.TABLE_FIELD ~ "x%"',
	'TABLE.TABLE_FIELD ~ "%%"');
$exp = '((TABLE.TABLE_FIELD = PREVIOUS.PREVIOUS_VAL) OR (TABLE.TABLE_FIELD = OTHER.PREVIOUS_OTHER)) AND ((TABLE.TABLE_FIELD LIKE "x%"))';
$res = &$test(\@conds,\@tables);
is ( $res, $exp, '__parse_conditions');
is ( $#tables, 2, '__parse_conditions');
is ( $tables[0], 'TABLE', '__parse_conditions' );
is ( $tables[1], 'PREVIOUS', '__parse_conditions' );
is ( $tables[2], 'OTHER', '__parse_conditions' );
BEGIN { $nbtest += 5; }

$test = \&FDN::Sql::DB_Access::__parse_insert;
my @fields;
@fields = ( 'T_A=12', 'T_B=13' );
$exp = ' ( T_A,T_B ) VALUES ( 12,13 )';
$res = &$test(\@fields,'T');
is ( $res, $exp, '__parse_insert');
BEGIN { $nbtest += 1; }

@fields = ( 'A=12', 'T_B=13' );
$res = &$test(\@fields,'T');
is ( $res, $exp, '__parse_insert');
BEGIN { $nbtest += 1; }

$test = \&FDN::Sql::DB_Access::__set_opt_arg;
$res = &$test('TABLE_FIELD','count',undef);
$exp = 'count(TABLE_FIELD)';
is ( $res, $exp, '__set_opt_arg');
$res = &$test('T_A,T_B,T_C,T_D','count','T_A');
$exp = 'count(T_A),T_B,T_C,T_D';
is ( $res, $exp, '__set_opt_arg');
$res = &$test($res,'max','T_D');
$exp = 'count(T_A),T_B,T_C,max(T_D)';
is ( $res, $exp, '__set_opt_arg');
$res = &$test($res,'sum','T_B');
$exp = 'count(T_A),sum(T_B),T_C,max(T_D)';
is ( $res, $exp, '__set_opt_arg');
$res = &$test($res,'min','T_C');
$exp = 'count(T_A),sum(T_B),min(T_C),max(T_D)';
is ( $res, $exp, '__set_opt_arg');
BEGIN { $nbtest += 5; }

$test = \&FDN::Sql::DB_Access::__add_query_opt;
$FDN::Sql::DB_Access::first_group = 'GROUP BY';
$res = &$test('group','T_E');
$exp = ' GROUP BY T_E';
is ( $res, $exp, '__add_query_opt');
$res = &$test('limit','12');
$exp = ' LIMIT 12';
is ( $res, $exp, '__add_query_opt');
$res = &$test('limit','10,5');
$exp = ' LIMIT 10,5';
is ( $res, $exp, '__add_query_opt');
$FDN::Sql::DB_Access::first_order = 'ORDER BY';
$res = &$test('orderup','T_F');
$exp = ' ORDER BY T_F ASC';
is ( $res, $exp, '__add_query_opt');
$FDN::Sql::DB_Access::first_order = 'ORDER BY';
$res = &$test('orderdn','T_F');
$exp = ' ORDER BY T_F DESC';
is ( $res, $exp, '__add_query_opt');
$FDN::Sql::DB_Access::first_order = 'ORDER BY';
$res = &$test('orderdown','T_F');
is ( $res, $exp, '__add_query_opt');
BEGIN { $nbtest += 6; }




$test = \&FDN::Sql::DB_Access::__parse_options;
my @options;
my $what;
my $table;
@options = ('count(T_A)', 'sum(T_B)', 'min(T_C)', 'max(T_D)',
	    'distinct', 'group(T_E)', 'group(T_A)', 'orderup(T_A)', 'orderdn(T_B)', 'limit(2,3)');
$what = 'T_A,T_B,T_C,T_D,O.O_X';
$table = 'T,O';
$cond  = 'T.T_X=O.O_X';
$exp = 'SELECT DISTINCT count(T_A),sum(T_B),min(T_C),max(T_D),O.O_X FROM T,O WHERE T.T_X=O.O_X GROUP BY T_E , T_A ORDER BY T_A ASC , T_B DESC LIMIT 2,3';
$res = &$test(\@options,$what,$table,$cond);
is ( $res, $exp, '__parse_options');
@options = ();
$what = 'T.*';
$table = 'T';
$cond = 'none';
$res = &$test(\@options,$what,$table,$cond);
$exp = 'SELECT T.* FROM T';
is ( $res, $exp, '__parse_options');
@options = ( undef, '' );
$res = &$test(\@options,$what,$table,$cond);
is ( $res, $exp, '__parse_options');
BEGIN { $nbtest += 3; }


$test = \&FDN::Sql::DB_Access::__build_sql_query;
@fields = ( 'T_A=12', 'T_B=13' );
@tables = ( 'T' );
@conds = ();
$exp = 'INSERT INTO T ( T_A,T_B ) VALUES ( 12,13 )';
$res = &$test('insert',\@fields,\@tables,\@conds,\@options);
is ( $res, $exp, '__build_sql_query/insert');
$res = &$test('insert',\@fields,\@tables,'Inutiles',\@options);
is ( $res, $exp, '__build_sql_query/insert');
$res = &$test('insert',\@fields,\@tables,'Inutiles','Bof');
is ( $res, $exp, '__build_sql_query/insert');
@conds = ( 'T.OTHER_VAL=12' );
$exp = 'UPDATE T SET T_A=12,T_B=13 WHERE ((T.OTHER_VAL = 12))';
$res = &$test('update',\@fields,\@tables,\@conds,\@options);
is ( $res, $exp, '__build_sql_query/update');
$res = &$test('update',\@fields,\@tables,\@conds,'Inutiles');
is ( $res, $exp, '__build_sql_query/update');
BEGIN { $nbtest += 5; }

$exp = 'DELETE FROM T WHERE ((T.OTHER_VAL = 12))';
$res = &$test('delete',\@fields,\@tables,\@conds,\@options);
is ( $res, $exp, '__build_sql_query/delete');
$res = &$test('delete',['prout prout caca'],\@tables,\@conds,\@options);
is ( $res, $exp, '__build_sql_query/delete');
$res = &$test('delete',['prout prout caca'],\@tables,\@conds,'Ca va te faire rire...');
is ( $res, $exp, '__build_sql_query/delete');
BEGIN { $nbtest += 3; }


@fields = ('T.*');
@tables = ('T');
@conds  = ('T_ID=12');
@options= ();
$res = &$test('select',\@fields,\@tables,\@conds,\@options);
$exp = "SELECT T.* FROM T WHERE ((T.T_ID = 12))";
is ( $res, $exp, '__build_sql_query/select');
BEGIN { $nbtest += 1; }

@fields = ('T_NOM');
@tables = ();
@conds  = ('T_NOM~"%%"', 'T_X=U_X');
@options = ( 'orderup(T_NOM)', 'orderdown(T_X)' );
$res = &$test('select',\@fields,\@tables,\@conds,\@options);
$exp = "SELECT T_NOM FROM T,U WHERE ((T.T_X = U.U_X)) ORDER BY T_NOM ASC , T_X DESC";
is ( $res, $exp, '__build_sql_query/select');
BEGIN { $nbtest += 1; }

@fields = ('T_NOM');
@tables = ();
@conds  = ('T_ID=U.T_ID','U_TYPE=\'toto\'');
@options = ();
$exp = "SELECT T_NOM FROM T,U WHERE ((T.T_ID = U.T_ID)) AND ((U.U_TYPE = 'toto'))";
$res = &$test('select',\@fields,\@tables,\@conds,\@options);
is ( $res, $exp, '__build_sql_query/select');
BEGIN { $nbtest += 1; }

@conds = ('T_ID=U.T_ID','U_TYPE=\'toto\' or U_TYPE=\'titi\'');
$exp = "SELECT T_NOM FROM T,U WHERE ((T.T_ID = U.T_ID)) AND ((U.U_TYPE = 'toto') OR (U.U_TYPE = 'titi'))";
$res = &$test('select',\@fields,\@tables,\@conds,\@options);
is ( $res, $exp, '__build_sql_query/select');
$exp = "DROP TABLE T";
$res = &$test('drop','rien',['T'],'rien','rien');
is ( $res, $exp, '__build_sql_query/drop');
BEGIN { $nbtest += 2; }

$test = \&FDN::Sql::DB_Access::__parse_data;
my @elems;
@elems = ('T');
@tables= ();
@fields= ();
&$test(\@elems,\@fields,\@tables);
is ( $#tables, 0, '__parse_data');
is ( $#fields, 0, '__parse_data');
is ( $tables[0], 'T', '__parse_data');
is ( $fields[0], 'T.*', '__parse_data');
BEGIN { $nbtest += 4; }

@elems = ('T_NOM','U','V.U_ID');
@tables= ();
@fields= ();
&$test(\@elems,\@fields,\@tables);
is_deeply( \@tables, ['T','U','V'], '__parse_data');
is_deeply( \@fields, ['T.T_NOM', 'U.*', 'V.U_ID'], '__parse_data');
BEGIN { $nbtest += 2; }

$test = \&FDN::Sql::DB_Access::__parse_update;
my @modifs;
@modifs = ( 'T.U_A=1', 'T_B=2', 'T.T_C=3' );
@tables = ();
@fields = ();
$res = &$test(\@modifs,\@fields,\@tables);
ok ( $res, '__parse_update');
is ( $#tables, 0, '__parse_update');
is ( $tables[0], 'T', '__parse_update');
is ( $#fields, 2, '__parse_update');
is_deeply ( \@fields, [ 'T.U_A=1', 'T_B=2', 'T.T_C=3' ], '__parse_update' );
BEGIN { $nbtest += 5; }


@modifs = ( 'T_A=1', 'U_B=2' );
@tables = ();
@fields = ();
$res = &$test(\@modifs,\@fields,\@tables);
ok ( ! $res, '__parse_update' );
$res = db_trace(0,'append','t/trace.log');
is ( $res, 0, 'db_trace');
$res = db_trace(1,'append','t/trace.log');
is ( $res, 0, 'db_trace');
$res = db_trace(0,'append','t/trace.log');
is ( $res, 1, 'db_trace');
BEGIN { $nbtest += 4; }

## 59
#$res = FDN::Sql::DB_Access::__get_script_name();
#like ( $res, qr/t\/15db-access.pl$/, '__get_script_name');
#
## 60
#FDN::Sql::DB_Access::init_dbaccess();
#like ( $FDN::Sql::DB_Access::fullname, qr/t\/15db-access.pl$/, 'init_dbaccess');
#
## 61
#ok ( defined $FDN::Sql::DB_Access::GET_DATA_REUSE->{$res}, 'init_dbaccess');
#
## 62
#is_deeply( $FDN::Sql::DB_Access::GET_DATA_REUSE->{$res}, {}, 'init_dbaccess');



my $skip_test;

SKIP: {

	if ( ! -f 't/dbc.cfg' ) {
		skip 'No mysql access', $skip_test;
	}
	use_ok('FDN::Common::Config');
	my $cfg = Load_Config('./t/dbc.cfg');
	is ( ref($cfg), 'HASH', 'dbc.cfg');
	BEGIN { $skip_test += 2; }

	my $dbh;
	$cfg = $cfg->{'test'};
	$dbh = connect_db($cfg->{'type'}, $cfg->{'host'}, $cfg->{'name'}, $cfg->{'user'},
		$cfg->{'passwd'} );
	is ( ref($dbh), 'HASH', 'connect_db' );
	ok ( disconnect_db($dbh), 'diconnect_db' );
	BEGIN { $skip_test += 2; }


	$dbh = connect_db($cfg->{'type'}, $cfg->{'host'}, $cfg->{'name'}, $cfg->{'user'},
		$cfg->{'passwd'} );
	$res = sql_transaction($dbh, 'drop', 'drop table if exists T');
	is ( ref($res), 'HASH', 'sql_transaction');
	is ( $res->{'ok'}, 1, 'sql_transaction');
	BEGIN { $skip_test += 2; }


	$res = sql_transaction($dbh, 'create', 'create table T ( T_A int, T_B int )');
	is ( ref($res), 'HASH', 'sql_transaction');
	ok ( ! $res->{'err_db'}, 'sql_transaction');
	is ( $res->{'ok'}, 1, 'sql_transaction');
	BEGIN { $skip_test += 3; }

	$res = sql_transaction($dbh,'insert','insert into T values(1,2)');
	is ( ref($res), 'HASH', 'sql_transaction');
	ok ( $res->{'ok'}, 'sql_transaction');
	BEGIN { $skip_test += 2; }

	$res = sql_transaction($dbh,'insert','insert into T values(3,4)');
	is ( ref($res), 'HASH', 'sql_transaction');
	ok ( $res->{'ok'}, 'sql_transaction');
	BEGIN { $skip_test += 2; }

	$res = sql_transaction($dbh,'select','select T.* from T order by T_A asc');
	is ( ref($res), 'HASH', 'sql_transaction');
	ok ( $res->{'ok'}, 'sql_transaction');
	is ( $res->{'count_rows'}, 2, 'sql_transaction');
	is ( ref($res->{'rows'}), 'ARRAY', 'sql_transaction');
	is ( $res->{'rows'}->[0]->{'T_A'}, 1, 'sql_transaction');
	is ( $res->{'rows'}->[0]->{'T_B'}, 2, 'sql_transaction');
	is ( $res->{'rows'}->[1]->{'T_A'}, 3, 'sql_transaction');
	is ( $res->{'rows'}->[1]->{'T_B'}, 4, 'sql_transaction');
	BEGIN { $skip_test += 8; }


	$res = get_data($dbh,['T_A'],[],['orderup(T_A)'],[]);
	is ( ref($res), 'HASH', 'get_data');
	is ( $res->{'count_rows'}, 2, 'get_data');
	is ( $res->{'rows'}->[0]->{'T_A'}, 1, 'get_data');
	is ( $res->{'rows'}->[1]->{'T_A'}, 3, 'get_data');
	is ( $res->{'rows'}->[0]->{'T_B'}, undef, 'get_data');
	BEGIN { $skip_test += 5; }


	$res = get_data($dbh,['T'],['T_A=1'],[],[]);
	is ( ref($res), 'HASH', 'get_data');
	ok ( $res->{'ok'}, 'get_data');
	is ( $res->{'count_rows'}, 1, 'get_data');
	BEGIN { $skip_test += 3; }

	$exp = 'SELECT T.* FROM T WHERE ((T.T_A = 1))';
	is ( $res->{'query'}, $exp, 'get_data');
	$res = new_data($dbh,['T'],['T_A=5','T_B=6']);
	is ( ref($res), 'HASH', 'new_data');
	ok ( $res->{'ok'}, 'new_data');
	$exp = 'INSERT INTO T ( T_A,T_B ) VALUES ( 5,6 )';
	is ( $res->{'query'}, $exp, 'new_data');
	BEGIN { $skip_test += 4; }


	$res = set_data($dbh,['T_B=7'],['T_A=5']);
	is ( ref($res), 'HASH', 'set_data');
	ok ( $res->{'ok'}, 'set_data' );
	BEGIN { $skip_test += 2; }

	$res = get_data($dbh,['T'],['T_A=5'],[],[]);
	is ( $res->{'rows'}->[0]->{'T_B'}, 7, 'set_data');
	BEGIN { $skip_test += 1; }

	sql_transaction($dbh,'drop','drop table if exists U');
	sql_transaction($dbh,'create','create table U ( U_A int, U_B int)');
	$res = lock_tables($dbh,['T WRITE']);
	ok ( $res->{'ok'}, 'lock_tables');
	$res = get_data($dbh,['U'],['U_A=12'],[],[]);
	ok ( ! $res->{'ok'}, 'lock_tables');
	$res = lock_tables($dbh,['U WRITE']);
	ok ( $res->{'ok'}, 'lock_tables');
	$res = get_data($dbh,['U'],['U_A=12'],[],[]);
	ok ( $res->{'ok'}, 'lock_tables');
	$res = unlock_tables($dbh,['U WRITE']);
	ok ( $res->{'ok'}, 'lock_tables');
	# This might work with some mysql versions!
	#$res = get_data($dbh,['U'],['U_A=12'],[],[]);
	#But this cannot, or mysql is definitively broken:
	$res = new_data($dbh,['U'],['U_A=5','U_B=6']);
	ok ( $res->{'err_db'}, 'lock_tables');
	$res = unlock_tables($dbh);
	ok ( $res->{'ok'}, 'lock_tables');
	$res = get_data($dbh,['U'],['U_A=12'],[],[]);
	ok ( $res->{'ok'}, 'lock_tables');
	BEGIN { $skip_test += 8; }


	$res = lock_soft($dbh,'toto');
	ok ( $res, 'lock_soft');
	ok ( is_locked_soft($dbh,'toto'), 'lock_soft');
	$res = sql_transaction($dbh,'select','select IS_USED_LOCK(\'toto\') as L');
	ok ( $res->{'ok'}, 'lock_soft');
	is ( $res->{'count_rows'}, 1, 'lock_soft' );
	isnt ( $res->{'rows'}->[0]->{'L'}, undef, 'lock_soft' );
	BEGIN { $skip_test += 5; }

	$res = lock_soft($dbh,'titi');
	ok ( ! $res, 'lock_soft' );
	ok ( ! is_locked_soft($dbh,'titi'), 'lock_soft');
	$res = sql_transaction($dbh,'select','select IS_USED_LOCK(\'toto\') as L');
	ok ( $res->{'ok'}, 'lock_soft' );
	isnt ( $res->{'rows'}->[0]->{'L'}, undef, 'lock_soft');
	$res = sql_transaction($dbh,'select','select IS_USED_LOCK(\'titi\') as L');
	ok ( $res->{'ok'}, 'lock_soft' );
	is ( $res->{'rows'}->[0]->{'L'}, undef, 'lock_soft' );
	BEGIN { $skip_test += 6; }

	$res = unlock_soft($dbh,'toto');
	ok ( $res, 'lock_soft');
	ok ( ! is_locked_soft($dbh,'toto'), 'lock_soft');
	ok ( ! is_locked_soft($dbh,'titi'), 'lock_soft');
	$res = sql_transaction($dbh,'select','select IS_USED_LOCK(\'titi\') as L');
	ok ( $res->{'ok'}, 'lock_soft' );
	is ( $res->{'rows'}->[0]->{'L'}, undef, 'lock_soft' );
	BEGIN { $skip_test += 5; }


	$res = unlock_soft($dbh,'titi');
	ok ( ! $res , 'lock_soft');
	ok ( ! is_locked_soft($dbh,'titi'), 'lock_soft');
	$res = is_locked_soft($dbh);
	is ( $res, '', 'lock_soft');
	BEGIN { $skip_test += 3; }

	$res = get_table_create($dbh,'T');
	$exp = '(T_A int, T_B int)';
	like ( $res, qr/^\s*\(\s*`T_A`\s*int\(11\)\s*default\s*NULL\s*,\s*`T_B`\s*int\(11\)\s*default\s*NULL\s*\)/im, 'get_table_create');
	BEGIN { $skip_test += 1; }


	sql_transaction($dbh,'drop','drop table if exists TEMP');
	$res = clone_table_to($dbh,'V','TEMP');
	ok ( ! $res, 'clone_table_to');
	sql_transaction($dbh,'drop','drop table if exists V');
	$res = clone_table_to($dbh,'T','V');
	ok ( $res, 'clone_table_to');
	$exp = get_table_create($dbh,'T');
	$res = get_table_create($dbh,'V');
	is ( $res, $exp, 'clone_table_to');
	BEGIN { $skip_test += 3; }


	sql_transaction($dbh,'drop','drop table if exists OLDT');
	$res = rename_tables($dbh, [{old=>'T',new=>'OLDT'},{old=>'V',new=>'T'}]);
	ok ( $res, 'rename_tables' );
	$res = get_table_create($dbh,'OLDT');
	is ( $res, $exp, 'rename_tables');
	BEGIN { $skip_test += 2; }


	$res = get_table_create($dbh,'V');
	is ( $res, undef, 'rename_tables');
	$res = get_data($dbh,['OLDT'],[],['orderdn(T_A)'],[]);
	ok ( $res->{'ok'}, 'rename_tables');
	is ( $res->{'count_rows'}, 3, 'rename_tables');
	is ( $res->{'rows'}->[0]->{'T_A'}, 5, 'rename_tables');
	is ( $res->{'rows'}->[1]->{'T_A'}, 3, 'rename_tables');
	is ( $res->{'rows'}->[2]->{'T_A'}, 1, 'rename_tables');
	BEGIN { $skip_test += 6; }


	$res = get_data($dbh,['OLDT'],['OLDT.T_A=5'],'','');
	is ( $res->{'count_rows'}, 1, 'del_data');
	$res = del_data($dbh,['OLDT'],['OLDT.T_A=5']);
	is ( ref($res), 'HASH', 'del_data');
	ok ( $res->{'ok'}, 'del_data');
	$res = get_data($dbh,['OLDT'],['OLDT.T_A=5'],'','');
	is ( $res->{'count_rows'}, 0, 'del_data');
	BEGIN { $skip_test += 4; }


	$test = \&FDN::Sql::DB_Access::__incr_sequence;
	$res = &$test(0,1,0,100,0);
	is ( $res, 1, '__incr_sequence');
	$res = &$test(12,-1,0,100,0);
	is ( $res, 11, '__incr_sequence');
	$res = &$test(0,-1,0,100,1);
	is ( $res, 100, '__incr_sequence');
	$res = &$test(100,1,0,100,1);
	is ( $res, 0, '__incr_sequence');
	BEGIN { $skip_test += 4; }


	$res = sql_transaction($dbh,'drop','drop table if exists SEQ');
	ok ( $res->{'ok'}, 'cleanup table SEQ');
	$res = sql_transaction($dbh,'create',
		"create table SEQ ( ".
			"SEQ_NAME varchar(20) UNIQUE NOT NULL default '', ".
			"SEQ_VALUE int(11) NOT NULL default '0', ".
			"SEQ_MINVALUE int(11) NOT NULL default '0', ".
			"SEQ_MAXVALUE int(11) NOT NULL default '0', ".
			"SEQ_INC int(11) NOT NULL default '0', ".
			"SEQ_CYCLE ENUM('-1','0','1') NOT NULL default '-1', ".
			"primary key  (SEQ_NAME) )");
	ok ( $res->{'ok'}, 'create table SEQ');
	BEGIN { $skip_test += 2; }

	$res = new_data($dbh,['SEQ'],[dochar('X_ID'),0,0,999999,1,dochar(0)]);
	ok ( $res->{'ok'}, 'create sequence X_ID');
	$res = get_data($dbh,['SEQ'],['SEQ_NAME='.dochar('X_ID')],'','');
	ok ( $res->{'ok'}, 'sequence_data');
	is ( $res->{'count_rows'}, 1, 'sequence_data');
	is ( $res->{'rows'}->[0]->{'SEQ_VALUE'}, 0, 'sequence_data');
	BEGIN { $skip_test += 4; }

	$res = new_data($dbh,['SEQ'],[dochar('S"eq'),0,0,999999,1,dochar(0)]);
	ok ( $res->{'ok'}, 'create sequence S"eq');
	BEGIN { $skip_test += 1; }

	$res = sequence_data($dbh,'X_ID');
	is ( $res, 1, 'sequence_data');
	$res = sequence_data($dbh,'X_ID');
	is ( $res, 2, 'sequence_data');
	$res = sequence_data($dbh,'X_ID','force',999999);
	is ( $res, 999999, 'sequence_data');
	$res = sequence_data($dbh,'X_ID');
	is ( $res, undef, 'sequence_data');
	BEGIN { $skip_test += 4; }

	$res = sequence_data($dbh,'X_ID','stats');
	is ( ref($res), 'HASH', 'sequence_data' );
	is ( $res->{'min'}, 0, 'sequence_data' );
	is ( $res->{'max'}, 999999, 'sequence_data' );
	is ( $res->{'free'}, 0, 'sequence_data' );
	is ( $res->{'busy'}, 999999, 'sequence_data' );
	is ( $res->{'ratio'}, 1, 'sequence_data' );
	BEGIN { $skip_test += 6; }

	$res = new_data($dbh,['SEQ'],[dochar('Y_ID'),0,0,3,1,dochar(1)]);
	ok ( $res->{'ok'}, 'create sequence Y_ID');
	$res = sequence_data($dbh,'Y_ID');
	is ( $res, 1, 'sequence_data');
	$res = sequence_data($dbh,'Y_ID');
	is ( $res, 2, 'sequence_data');
	$res = sequence_data($dbh,'Y_ID');
	is ( $res, 3, 'sequence_data');
	$res = sequence_data($dbh,'Y_ID');
	is ( $res, 0, 'sequence_data');
	BEGIN { $skip_test += 5; }


	$test = \&FDN::Sql::DB_Access::__parse_conditions;
	my @values;
	@tables = ();
	@conds = (
		'TABLE_FIELD = PREVIOUS_VAL or TABLE_FIELD = OTHER.PREVIOUS_OTHER',
		'TABLE.TABLE_FIELD ~ "x%"',
		'TABLE.TABLE_FIELD ~ "%%"');
	$exp = '((TABLE.TABLE_FIELD = PREVIOUS.PREVIOUS_VAL) OR (TABLE.TABLE_FIELD = OTHER.PREVIOUS_OTHER)) AND ((TABLE.TABLE_FIELD LIKE ?))';
	$res = &$test(\@conds,\@tables,\@values);
	is ( $res, $exp, '__parse_conditions/reuse');
	is ( $#values, 0, '__parse_conditions/reuse');
	is_deeply( \@values, ['x%'], '__parse_conditions/reuse');
	BEGIN { $skip_test += 3; }


	$res = get_data_reuse($dbh,['SEQ'],['SEQ_NAME='.dochar('X_ID')],'','');
	ok ( $res->{'ok'}, 'get_data_reuse' );
	is ( $res->{'count_rows'}, 1, 'get_data_reuse');
	is ( $res->{'rows'}->[0]->{'SEQ_MAXVALUE'}, 999999, 'get_data_reuse');
	BEGIN { $skip_test += 3; }
	
	$res = get_data_reuse($dbh,['SEQ'],['SEQ_NAME='.dochar('S"eq')],'','');
	ok ( $res->{'ok'}, 'get_data_reuse' );
	is ( $res->{'count_rows'}, 1, 'get_data_reuse');
	is ( $res->{'rows'}->[0]->{'SEQ_MAXVALUE'}, 999999, 'get_data_reuse');
	BEGIN { $skip_test += 3; }
	
	my $sql;
	$sql = open_sql($dbh,['SEQ'],[],['orderup(SEQ_NAME)'],undef);
	is ( ref($sql), 'HASH', 'open_sql');
	is ( $sql->{'count_rows'}, 0, 'open_sql');
	$res = read_sql($sql);
	is ( ref($res), 'HASH', 'read_sql');
	is ( $res->{'SEQ_NAME'}, 'S"eq', 'read_sql');
	is ( $res->{'SEQ_MAXVALUE'}, 999999, 'read_sql');
	$res = read_sql($sql);
	is ( ref($res), 'HASH', 'read_sql');
	is ( $res->{'SEQ_NAME'}, 'X_ID', 'read_sql');
	is ( $res->{'SEQ_MAXVALUE'}, 999999, 'read_sql');
	$res = read_sql($sql);
	is ( ref($res), 'HASH', 'read_sql');
	is ( $res->{'SEQ_NAME'}, 'Y_ID', 'read_sql');
	is ( $res->{'SEQ_MAXVALUE'}, 3, 'read_sql');
	$res = read_sql($sql);
	is ( $res, undef, 'read_sql');
	close_sql($sql);
	is ( $dbh->{'Verrou'}, 0, 'close_sql');
	BEGIN { $skip_test += 13; }


	$sql = open_sql($dbh,['SEQ'],[],['orderup(SEQ_NAME)'],undef);
	is ( ref($sql), 'HASH', 'open_sql');
	is ( $sql->{'count_rows'}, 0, 'open_sql');
	$res = open_sql($dbh,['T'],[],[],[]);
	is ( $res, undef, 'open_sql');
	$res = get_data($dbh,['T'],[],[],[]);
	is ( ref($res), 'HASH', 'open_sql');
	ok ( ! $res->{'ok'}, 'open_sql');
	close_sql($sql);
	BEGIN { $skip_test += 5; }

	my $new_dbh = clone_dbh($dbh);
	is ( ref($new_dbh), 'HASH', 'clone_dbh' );
	$res = get_data_reuse($dbh,['SEQ'],['SEQ_NAME='.dochar('X_ID')],'','');
	ok ( $res->{'ok'}, 'clone_dbh' );
	BEGIN { $skip_test += 2; }

	$sql = open_sql($new_dbh,['SEQ'],[],['orderup(SEQ_NAME)'],undef);
	is ( ref($sql), 'HASH', 'open_sql');
	is ( $sql->{'count_rows'}, 0, 'open_sql');
	$res = open_sql($new_dbh,['T'],[],[],[]);
	is ( $res, undef, 'open_sql');
	$res = get_data($dbh,['T'],[],[],[]);
	is ( ref($res), 'HASH', 'open_sql');
	ok ( $res->{'ok'}, 'open_sql' );
	close_sql($sql);
	disconnect_db($new_dbh);
	BEGIN { $skip_test += 5; }

	$res = get_data($dbh,['T'],[],[],[]);
	is ( ref($res), 'HASH', 'clone_dbh');
	ok ( $res->{'ok'}, 'clone_dbh' );
	BEGIN { $skip_test += 2; }
	

	# Clean up
	sql_transaction($dbh,'drop','drop table if exists TEMP');
	sql_transaction($dbh,'drop','drop table if exists T');
	sql_transaction($dbh,'drop','drop table if exists U');
	sql_transaction($dbh,'drop','drop table if exists V');
	sql_transaction($dbh,'drop','drop table if exists OLDT');
	sql_transaction($dbh,'drop','drop table if exists SEQ');
	
} 

BEGIN { $nbtest += $skip_test; }

BEGIN { plan tests => $nbtest+1; }
BEGIN { use_ok('FDN::Sql::DB_Access'); }

